@@ -1,5 +1,5 @@
 package experimental;
-$experimental::VERSION = '0.028';
+$experimental::VERSION = '0.030';
 use strict;
 use warnings;
 use version ();
@@ -8,6 +8,7 @@ BEGIN { eval { require feature } };
 use Carp qw/croak carp/;
 
 my %warnings = map { $_ => 1 } grep { /^experimental::/ } keys %warnings::Offsets;
+my %removed_warnings = map { $_ => 1 } grep { /^experimental::/ } keys %warnings::NoOp;
 my %features = map { $_ => 1 } $] > 5.015006 ? keys %feature::feature : do {
 	my @features;
 	if ($] >= 5.010) {
@@ -73,6 +74,9 @@ sub _enable {
 		feature->import($pragma);
 		_enable(@{ $additional{$pragma} }) if $additional{$pragma};
 	}
+	elsif ($removed_warnings{"experimental::$pragma"}) {
+		_enable(@{ $additional{$pragma} }) if $additional{$pragma};
+	}
 	elsif (not exists $min_version{$pragma}) {
 		croak "Can't enable unknown feature $pragma";
 	}
@@ -136,7 +140,7 @@ experimental - Experimental features made easy
 
 =head1 VERSION
 
-version 0.028
+version 0.030
 
 =head1 SYNOPSIS
 
@@ -251,10 +255,14 @@ This was added in perl 5.20.0.
 This was added in perl 5.10.0, but it should be noted there are significant
 incompatibilities between 5.10.0 and 5.10.1.
 
+The feature is going to be deprecated in perl 5.38.0, and removed in 5.42.0.
+
 =item * C<switch> - allow the use of C<~~>, given, and when
 
 This was added in perl 5.10.0.
 
+The feature is going to be deprecated in perl 5.38.0, and removed in 5.42.0.
+
 =item * C<try> - allow the use of C<try> and C<catch>
 
 This was added in perl 5.34.0
