@@ -83,8 +83,12 @@ Perl_debug_studydata(pTHX_ const char *where, scan_data_t *data,
     DECLARE_AND_GET_RE_DEBUG_FLAGS;
 
     DEBUG_OPTIMISE_MORE_r({
-        if (!data)
+        if (!data) {
+            Perl_re_indentf(aTHX_  "%s: NO DATA",
+                depth,
+                where);
             return;
+        }
         Perl_re_indentf(aTHX_  "%s: M/S/D: %" IVdf "/%" IVdf "/%" IVdf " Pos:%" IVdf "/%" IVdf " Flags: 0x%" UVXf,
             depth,
             where,
@@ -170,7 +174,7 @@ S_regdump_intflags(pTHX_ const char *lead, const U32 flags)
 
     ASSUME(REG_INTFLAGS_NAME_SIZE <= sizeof(flags)*8);
 
-    for (bit=0; bit<REG_INTFLAGS_NAME_SIZE; bit++) {
+    for (bit=0; bit<=REG_INTFLAGS_NAME_SIZE; bit++) {
         if (flags & (1<<bit)) {
             if (!set++ && lead)
                 Perl_re_printf( aTHX_  "%s", lead);
@@ -478,7 +482,6 @@ Perl_regprop(pTHX_ const regexp *prog, SV *sv, const regnode *o, const regmatch_
         U32 parno= (op == ACCEPT)              ? (U32)ARG2L(o) :
                    (op == OPEN || op == CLOSE) ? (U32)PARNO(o) :
                                                  (U32)ARG(o);
-        Perl_sv_catpvf(aTHX_ sv, "%" UVuf, (UV)parno);        /* Parenth number */
         if ( RXp_PAREN_NAMES(prog) ) {
             name_list= MUTABLE_AV(progi->data->data[progi->name_list_idx]);
         } else if ( pRExC_state ) {
@@ -486,6 +489,14 @@ Perl_regprop(pTHX_ const regexp *prog, SV *sv, const regnode *o, const regmatch_
         }
         if ( name_list ) {
             if ( k != REF || (op < REFN)) {
+                UV logical_parno = parno;
+                if (prog->parno_to_logical)
+                    logical_parno = prog->parno_to_logical[parno];
+
+                Perl_sv_catpvf(aTHX_ sv, "%" UVuf, (UV)logical_parno);     /* Parenth number */
+                if (parno != logical_parno)
+                    Perl_sv_catpvf(aTHX_ sv, "/%" UVuf, (UV)parno);        /* Parenth number */
+
                 SV **name= av_fetch_simple(name_list, parno, 0 );
                 if (name)
                     Perl_sv_catpvf(aTHX_ sv, " '%" SVf "'", SVfARG(*name));
@@ -511,23 +522,36 @@ Perl_regprop(pTHX_ const regexp *prog, SV *sv, const regnode *o, const regmatch_
                     Perl_sv_catpvf(aTHX_ sv, " '%" SVf "'", SVfARG(*name));
                 }
             }
+        } else if (parno>0) {
+            UV logical_parno = parno;
+            if (prog->parno_to_logical)
+                logical_parno = prog->parno_to_logical[parno];
+
+            Perl_sv_catpvf(aTHX_ sv, "%" UVuf, (UV)logical_parno);     /* Parenth number */
+            if (logical_parno != parno)
+                Perl_sv_catpvf(aTHX_ sv, "/%" UVuf, (UV)parno);     /* Parenth number */
+
         }
         if ( k == REF && reginfo) {
             U32 n = ARG(o);  /* which paren pair */
-            I32 ln = prog->offs[n].start;
-            if (prog->lastparen < n || ln == -1 || prog->offs[n].end == -1)
+            I32 ln = RXp_OFFS_START(prog,n);
+            if (prog->lastparen < n || ln == -1 || RXp_OFFS_END(prog,n) == -1)
                 Perl_sv_catpvf(aTHX_ sv, ": FAIL");
-            else if (ln == prog->offs[n].end)
+            else if (ln == RXp_OFFS_END(prog,n))
                 Perl_sv_catpvf(aTHX_ sv, ": ACCEPT - EMPTY STRING");
             else {
                 const char *s = reginfo->strbeg + ln;
                 Perl_sv_catpvf(aTHX_ sv, ": ");
-                Perl_pv_pretty( aTHX_ sv, s, prog->offs[n].end - prog->offs[n].start, 32, 0, 0,
+                Perl_pv_pretty( aTHX_ sv, s, RXp_OFFS_END(prog,n) - RXp_OFFS_START(prog,n), 32, 0, 0,
                     PERL_PV_ESCAPE_UNI_DETECT|PERL_PV_PRETTY_NOCLEAR|PERL_PV_PRETTY_ELLIPSES|PERL_PV_PRETTY_QUOTE );
             }
         }
     } else if (k == GOSUB) {
         AV *name_list= NULL;
+        IV parno = ARG(o);
+        IV logical_parno = (parno && prog->parno_to_logical)
+                         ? prog->parno_to_logical[parno]
+                         : parno;
         if ( RXp_PAREN_NAMES(prog) ) {
             name_list= MUTABLE_AV(progi->data->data[progi->name_list_idx]);
         } else if ( pRExC_state ) {
@@ -535,7 +559,11 @@ Perl_regprop(pTHX_ const regexp *prog, SV *sv, const regnode *o, const regmatch_
         }
 
         /* Paren and offset */
-        Perl_sv_catpvf(aTHX_ sv, "%d[%+d:%d]", (int)ARG(o),(int)ARG2L(o),
+        Perl_sv_catpvf(aTHX_ sv, "%" IVdf, logical_parno);
+        if (logical_parno != parno)
+            Perl_sv_catpvf(aTHX_ sv, "/%" IVdf, parno);
+
+        Perl_sv_catpvf(aTHX_ sv, "[%+d:%d]", (int)ARG2L(o),
                 (int)((o + (int)ARG2L(o)) - progi->program) );
         if (name_list) {
             SV **name= av_fetch_simple(name_list, ARG(o), 0 );
@@ -843,6 +871,10 @@ Perl_regprop(pTHX_ const regexp *prog, SV *sv, const regnode *o, const regmatch_
     }
     else if (op == SBOL)
         Perl_sv_catpvf(aTHX_ sv, " /%s/", o->flags ? "\\A" : "^");
+    else if (op == EVAL) {
+        if (o->flags & EVAL_OPTIMISTIC_FLAG)
+            Perl_sv_catpvf(aTHX_ sv, " optimistic");
+    }
 
     /* add on the verb argument if there is one */
     if ( ( k == VERB || op == ACCEPT || op == OPFAIL ) && o->flags) {
