@@ -647,6 +647,7 @@ h_fcntl='false'
 h_sysfile='true'
 i16size='2'
 i16type='short'
+i32dformat='"ld"'
 i32size='4'
 i32type='long'
 i64size='8'
@@ -875,8 +876,12 @@ touch='touch'
 ttyname_r_proto='0'
 u16size='2'
 u16type='unsigned short'
+u32XUformat='"lX"'
+u32oformat='"lo"'
 u32size='4'
 u32type='unsigned long'
+u32uformat='"lu"'
+u32xformat='"lx"'
 u64size='8'
 u64type='uint64_t'
 u8size='1'
